from enum import Enum


class VirtualRemoteKey(Enum):
    """Keys to be used with the virtual remote"""

    KEY_SOURCE = 0x01
    KEY_POWER = 0x02
    KEY_1 = 0x04
    KEY_2 = 0x05
    KEY_3 = 0x06
    KEY_VOLUME_UP = 0x07
    KEY_4 = 0x08
    KEY_5 = 0x09
    KEY_6 = 0x0A
    KEY_VOLUME_DOWN = 0x0B
    KEY_7 = 0x0C
    KEY_8 = 0x0D
    KEY_9 = 0x0E
    KEY_MUTE = 0x0F
    KEY_CHANNEL_DOWN = 0x10
    KEY_0 = 0x11
    KEY_CHANNEL_UP = 0x12
    KEY_GREEN = 0x14
    KEY_YELLOW = 0x15
    KEY_CYAN = 0x16
    KEY_MENU = 0x1A
    KEY_DISPLAY = 0x1F
    KEY_INFO = 0x1F
    KEY_DIGIT = 0x23
    KEY_PIP_TV_VIDEO = 0x24
    KEY_BLANK = 0x24
    KEY_EXIT = 0x2D
    KEY_MAGICINFO = 0x30
    KEY_REW = 0x45
    KEY_STOP = 0x46
    KEY_PLAY = 0x47
    KEY_FF = 0x48
    KEY_PAUSE = 0x4A
    KEY_TOOLS = 0x4B
    KEY_RETURN = 0x58
    KEY_MAGICINFO_LITE = 0x5B
    KEY_CURSOR_UP = 0x60
    KEY_CURSOR_DOWN = 0x61
    KEY_CURSOR_RIGHT = 0x62
    KEY_CURSOR_LEFT = 0x65
    KEY_ENTER = 0x68
    KEY_RED = 0x6C
    KEY_LOCK = 0x77
    KEY_CONTENT = 0x79
    KEY_POWER_OFF = 0x98
    KEY_3D = 0x9F



from enum import Enum


class InputSource(Enum):
    """ Input Sources """

    S_VIDEO = 0x04
    COMPONENT = 0x08
    AV = 0x0C
    AV_2 = 0x0D
    SCART = 0x0E
    DVI = 0x18
    PC = 0x14
    BNC = 0x1E
    DVI_VIDEO = 0x1F
    MAGICINFO = 0x20
    HDMI_1 = 0x21
    HDMI_1_PC = 0x22  # get only!
    HDMI_2 = 0x23
    HDMI_2_PC = 0x24  # get only!
    DP = 0x25
    DP_2 = 0x26
    DP_3 = 0x27
    HDMI_3 = 0x31
    HDMI_3_PC = 0x32  # get only!
    HDMI_4 = 0x33
    HDMI_4_PC = 0x34  # get only!
    TV = 0x40
    MODULE = 0x50
    HDBT = 0x55
    OCM = 0x56
    MEDIA = 0x60
    SCREEN_MIRRORING = 0x61
    INTERNAL_USB = 0x62
    URL_LAUNCHER = 0x63
    IWB = 0x64
    WEB_BROWSER = 0x65
    REMOTE_WORKSPACE = 0x65

    ALL_SOURCES = [
        S_VIDEO,
        COMPONENT,
        AV,
        AV_2,
        SCART,
        DVI,
        PC,
        BNC,
        DVI_VIDEO,
        MAGICINFO,
        HDMI_1,
        HDMI_1_PC,
        HDMI_2,
        HDMI_2_PC,
        DP,
        DP_2,
        DP_3,
        HDMI_3,
        HDMI_3_PC,
        HDMI_4,
        HDMI_4_PC,
        TV,
        MODULE,
        HDBT,
        OCM,
        MEDIA,
        SCREEN_MIRRORING,
        INTERNAL_USB,
        URL_LAUNCHER,
        IWB,
        WEB_BROWSER,
        REMOTE_WORKSPACE,
    ]

    ALL_SOURCE_NAMES = {
        S_VIDEO: "S-Video",
        COMPONENT: "Component",
        AV: "AV1 (AV)",
        AV_2: "AV2",
        SCART: "Ext. (SCART1)",
        DVI: "DVI",
        PC: "PC",
        BNC: "BNC",
        DVI_VIDEO: "DVI VIDEO",
        MAGICINFO: "MagicInfo",
        HDMI_1: "HDMI 1",
        HDMI_1_PC: "HDMI 1 (PC)",
        HDMI_2: "HDMI 2",
        HDMI_2_PC: "HDMI 2 (PC)",
        DP: "DisplayPort",
        DP_2: "DisplayPort 2",
        DP_3: "DisplayPort 3",
        HDMI_3: "HDMI 3",
        HDMI_3_PC: "HDMI 3 (PC)",
        HDMI_4: "HDMI 4",
        HDMI_4_PC: "HDMI 4 (PC)",
        TV: "TV (DTV)",
        MODULE: "Plug In Module",
        HDBT: "HDBaseT",
        OCM: "OCM",
        MEDIA: "Meida/MagicInfo S",
        SCREEN_MIRRORING: "WiDi/Screen Mirroring",
        INTERNAL_USB: "Internal/USB",
        URL_LAUNCHER: "URL launcher",
        IWB: "IWB",
        WEB_BROWSER: "Web Browser",
        REMOTE_WORKSPACE: "Remote Workspace",
    }



class AccessDenied(Exception):
    """Connection was denied."""

    pass


class ConnectionClosed(Exception):
    """Connection was closed."""

    pass


class UnhandledResponse(Exception):
    """Received unknown response."""

    pass


class InvalidResponse(Exception):
    """Received an invalid response (not an ack/nack)."""

    pass


class InvalidResponseDeviceId(Exception):
    """Received a response for some other device id."""

    pass


class NotEnoughData(Exception):
    """Received not enough data."""

    pass


class InvalidResponseChecksum(Exception):
    """Response checksum does not match payload."""

    pass


class NakReceived(Exception):
    """Nak received from the display"""

    pass


class UnknownMethod(Exception):
    """Unknown method."""

    pass



from samsung_mdc_ctl.helpers.panel_type import PanelType


class DisplayModel:
    def __init__(self, payload: bytes) -> None:

        self.panelType = PanelType(payload[0])
        self.model = payload[1]
        self.support_tv = bool(payload[2] == 0)



from typing import List

import logging
import socket
import time

import samsung_mdc_ctl.helpers.exceptions as exceptions
from samsung_mdc_ctl import utils
from samsung_mdc_ctl.protocol.commands import Command
from samsung_mdc_ctl.protocol.response import AckResponse, NakResponse, Response


class MDCConnection:
    """Object for the connection to the screen"""

    def __init__(self, host: str, deviceId: int, timeout: int = 10):
        self.connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        self.connection.settimeout(timeout)

        self.connection.connect((host, 1515))
        self.deviceId = deviceId & 0xFF

        # self._read_response(True)

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def close(self):
        """Close the connection."""
        if self.connection:
            self.connection.close()
            # self.connection = None
            logging.debug("Connection closed.")

    def send(self, cmd: Command, data: List[int] = []) -> Response:
        logging.info("Sending control command: %s", cmd)
        return self.sendRaw(rawCmd=cmd.value, data=data)

    def sendRaw(self, rawCmd: int, data: List[int] = []) -> Response:
        """Send a control command."""
        if not self.connection:
            raise exceptions.ConnectionClosed()

        # print(f"command: {cmd}")

        payload = bytearray()
        payload.append(rawCmd)
        payload.append(self.deviceId)
        payload += self._serialize_data(data)
        # print(utils.byte_compute_checksum(payload))

        packet = bytearray([0xAA])
        packet += payload
        packet.append(utils.compute_checksum(payload))
        # print(packet)

        logging.info("Sending command: %s", packet)
        self.connection.send(packet)
        return self._read_response()

    def _read_response(self) -> Response:
        read_data = self.connection.recv(1024)

        if len(read_data) <= 2:
            raise exceptions.NotEnoughData()

        if (
            read_data[0] != 0xAA
            or Command(read_data[1]) is not Command.ACK_NACK
        ):
            raise exceptions.InvalidResponse()

        checksum = read_data[-1]
        computed_checksum = utils.compute_checksum(read_data[1:-1])
        if checksum != computed_checksum:
            raise exceptions.InvalidResponseChecksum()

        if read_data[2] != self.deviceId:
            print("Received an response not for my device")
            raise exceptions.InvalidResponseDeviceId()

        dataLength = read_data[3]
        payload = read_data[4:-1]
        if len(payload) != dataLength:
            raise exceptions.NotEnoughData()

        rCmd = Command(payload[1])

        if payload[0] == ord("A"):
            return AckResponse(rCmd, payload[2:])
        elif payload[0] == ord("N"):
            return NakResponse(rCmd, payload[2])
        else:
            raise exceptions.UnhandledResponse()

    @staticmethod
    def _serialize_data(data):
        serialized_data = bytearray()
        serialized_data.append(len(data))
        serialized_data += bytearray(data)
        return serialized_data



from samsung_mdc_ctl.protocol.commands import Command


class Response:
    def __init__(self, rCmd: Command):
        self.rCmd = rCmd


class AckResponse(Response):
    def __init__(self, rCmd: Command, payload: bytes):
        super().__init__(rCmd)
        self.payload = payload


class NakResponse(Response):
    def __init__(self, rCmd: Command, errCode: int):
        super().__init__(rCmd)
        self.errCode = errCode



import logging
import time

from samsung_mdc_ctl.helpers.connection import MDCConnection
from samsung_mdc_ctl.helpers.display_status import DisplayStatus
from samsung_mdc_ctl.helpers.exceptions import NakReceived, UnhandledResponse
from samsung_mdc_ctl.helpers.model import DisplayModel
from samsung_mdc_ctl.helpers.status import Status
from samsung_mdc_ctl.protocol.commands import Command
from samsung_mdc_ctl.protocol.power import DisplayPower
from samsung_mdc_ctl.protocol.response import AckResponse, NakResponse, Response
from samsung_mdc_ctl.protocol.sources import InputSource
from samsung_mdc_ctl.protocol.virtual_remote_keys import VirtualRemoteKey


class MDCDisplay:
    """Object representing the MDC controlled display"""

    def __init__(self, host: str, deviceId: int) -> None:
        self._host = host
        self._deviceId = deviceId
        self.connection = MDCConnection(host=host, deviceId=deviceId)

    def _check_response(self, response: Response) -> AckResponse:
        if isinstance(response, AckResponse):
            return response
        elif isinstance(response, NakResponse):
            raise NakReceived(response.errCode)

    def getStatus(self) -> DisplayStatus:
        response = self.connection.send(Command.STATUS)
        statusResponse = self._check_response(response)

        if statusResponse.rCmd == Command.STATUS:
            return Status(statusResponse.payload)
        raise UnhandledResponse()

    def getSerialNumber(self) -> str:
        response = self.connection.send(Command.GET_SERIAL)
        serialResponse = self._check_response(response)

        if serialResponse.rCmd == Command.GET_SERIAL:
            return str(serialResponse.payload.decode("utf-8"))
        raise UnhandledResponse()

    def getDisplayStatusControl(self) -> DisplayStatus:
        response = self.connection.send(Command.GET_DISPLAY_STATUS)
        dispStatusResponse = self._check_response(response)

        if dispStatusResponse.rCmd == Command.GET_DISPLAY_STATUS:
            return DisplayStatus(dispStatusResponse.payload)
        raise UnhandledResponse()

    def getSWVersion(self) -> str:
        response = self.connection.send(Command.GET_SW_VER)
        swVerResponse = self._check_response(response)

        if swVerResponse.rCmd == Command.GET_SW_VER:
            return str(swVerResponse.payload.decode("utf-8"))
        raise UnhandledResponse()

    def getModelNumber(self) -> DisplayModel:
        response = self.connection.send(Command.GET_MODEL)
        modelResponse = self._check_response(response)

        if modelResponse.rCmd == Command.GET_MODEL:
            return DisplayModel(modelResponse.payload)
        raise UnhandledResponse()

    ############## 2.1.11 Power Control ##############

    def getPower(self) -> DisplayPower:
        response = self.connection.send(Command.POWER)
        pwrResponse = self._check_response(response)

        if pwrResponse.rCmd == Command.POWER:
            return DisplayPower(pwrResponse.payload[0])
        raise UnhandledResponse()

    # todo: on power on, wait 10 sec, then reconnect
    def setPower(self, power: DisplayPower) -> None:
        pwrResponse = self.connection.send(Command.POWER, [power.value])
        self._check_response(pwrResponse)
        if power == DisplayPower.POWER_ON:
            time.sleep(10)
            logging.debug("Reconnecting after power on")
            self.reconnect()
            logging.debug("Reconnected after power on")

    def reconnect(self):
        self.connection.close()
        reconnect_try_count = 3
        for i in range(reconnect_try_count):
            try:
                self.connection = MDCConnection(
                    host=self._host, deviceId=self._deviceId
                )
                return
            except:
                logging.info(
                    f"Reconnection failed, retrying ({i+1}/{reconnect_try_count})"
                )
                time.sleep(1)
        raise ConnectionRefusedError()

    ############## 2.1.12 Volume Control ##############

    def getVolume(self) -> int:
        response = self.connection.send(Command.VOLUME)
        volResponse = self._check_response(response)

        if volResponse.rCmd == Command.VOLUME:
            return int(volResponse.payload[0])
        raise UnhandledResponse()

    def setVolume(self, volume: int) -> None:
        volume = max(0, min(volume, 100))
        volResponse = self.connection.send(Command.VOLUME, [volume])
        self._check_response(volResponse)

    ############## 2.1.13 Mute Control ##############

    def getMute(self) -> bool:
        response = self.connection.send(Command.MUTE)
        muteResponse = self._check_response(response)

        if muteResponse.rCmd == Command.MUTE:
            return bool(muteResponse.payload[0] == 1)
        raise UnhandledResponse()

    def setMute(self, mute: bool) -> None:
        muteResponse = self.connection.send(Command.MUTE, [mute])
        self._check_response(muteResponse)

    ############## 2.1.14 Input Source Control ##############

    def getInputSource(self) -> InputSource:
        response = self.connection.send(Command.INPUT_SOURCE)
        inputSourceResponse = self._check_response(response)

        if inputSourceResponse.rCmd == Command.INPUT_SOURCE:
            return InputSource(inputSourceResponse.payload[0])
        raise UnhandledResponse()

    def setInputSource(self, src: InputSource) -> None:
        changeInputSourceResponse = self.connection.send(
            Command.INPUT_SOURCE, [src.value]
        )
        self._check_response(changeInputSourceResponse)

    ############## 2.1.B0 Virtual Remote Control ##############

    def sendRemoteKey(self, key: VirtualRemoteKey) -> None:
        sendRemoteKeyResponse = self.connection.send(
            Command.VIRTUAL_REMOTE, [key.value]
        )
        self._check_response(sendRemoteKeyResponse)



from enum import Enum


class Command(Enum):
    STATUS = 0x00
    GET_SERIAL = 0x0B
    GET_SW_VER = 0x0E
    GET_MODEL = 0x10
    POWER = 0x11
    VOLUME = 0x12
    MUTE = 0x13
    INPUT_SOURCE = 0x14
    CHANNEL_CONTROL = 0x17
    VIRTUAL_REMOTE = 0xB0
    PIP_STATUS = 0x09

    GET_DISPLAY_STATUS = 0x0D

    ACK_NACK = 0xFF



from enum import Enum


class LampError(Enum):
    NORMAL = 0x00
    ERROR = 0x01


class TemperatureError(Enum):
    NORMAL = 0x00
    ERROR = 0x01


class BrightSensorError(Enum):
    NONE = 0x00
    ERROR = 0x01
    NORMAL = 0x02


class NoSyncError(Enum):
    NORMAL = 0x00
    NO_SYNC = 0x01
    INVALID = 0x02


class FanError(Enum):
    NORMAL = 0x00
    ERROR = 0x01
    NOT_SUPPORTED = 0x02


class DisplayStatus:
    """Display status response object"""

    def __init__(self, payload: bytes) -> None:
        self.lampError = LampError(payload[0])
        self.tempError = TemperatureError(payload[1])
        self.brightSensorError = BrightSensorError(payload[2])
        self.noSyncError = NoSyncError(payload[3])
        self.temperature = int(payload[4])
        self.fanError = FanError(payload[5])
        pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['samsung_mdc_ctl', 'samsung_mdc_ctl.helpers', 'samsung_mdc_ctl.protocol']

package_data = \
{'': ['*']}

install_requires = \
['rich>=9.8.2,<10.0.0', 'typer[all]>=0.3.2,<0.4.0']

extras_require = \
{':python_version < "3.8"': ['importlib_metadata>=3.7.3,<4.0.0']}

entry_points = \
{'console_scripts': ['samsung-mdc-ctl = samsung_mdc_ctl.__main__:app']}

setup_kwargs = {
    'name': 'samsung-mdc-ctl',
    'version': '0.2.0',
    'description': 'Awesome `samsung-mdc-ctl` is a Python cli/package created with https://github.com/TezRomacH/python-package-template',
    'long_description': '# samsung-mdc-ctl\n\n<div align="center">\n\n[![Build status](https://github.com/matthijs2704/samsung-mdc-ctl/workflows/build/badge.svg?branch=main&event=push)](https://github.com/matthijs2704/samsung-mdc-ctl/actions?query=workflow%3Abuild)\n[![Python Version](https://img.shields.io/pypi/pyversions/samsung-mdc-ctl.svg)](https://pypi.org/project/samsung-mdc-ctl/)\n[![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/matthijs2704/samsung-mdc-ctl/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot)\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Security: bandit](https://img.shields.io/badge/security-bandit-green.svg)](https://github.com/PyCQA/bandit)\n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/matthijs2704/samsung-mdc-ctl/blob/main/.pre-commit-config.yaml)\n[![Semantic Versions](https://img.shields.io/badge/%F0%9F%9A%80-semantic%20versions-informational.svg)](https://github.com/matthijs2704/samsung-mdc-ctl/releases)\n[![License](https://img.shields.io/github/license/matthijs2704/samsung-mdc-ctl)](https://github.com/matthijs2704/samsung-mdc-ctl/blob/main/LICENSE)\n\nClean and full(ler)-featured Python package for controlling Samsung Displays using the Multiple Display Control (MDC) Protocol over IP.\n\n### :rotating_light: Work-In-Progress, breaking changes will happen :rotating_light:\n\n</div>\n\n## Installation\n\n```bash\npip install -U samsung-mdc-ctl\n```\n\nor install with `Poetry`\n\n```bash\npoetry add samsung-mdc-ctl\n```\n\nThen you can run\n\n```bash\nsamsung-mdc-ctl --help\n```\n\n```bash\nsamsung-mdc-ctl --name Roman\n```\n\nor if installed with `Poetry`:\n\n```bash\npoetry run samsung-mdc-ctl --help\n```\n\n```bash\npoetry run samsung-mdc-ctl --name Roman\n```\n\n### Makefile usage\n\n[`Makefile`](https://github.com/matthijs2704/samsung-mdc-ctl/blob/main/Makefile) contains many functions for fast assembling and convenient work.\n\n<details>\n<summary>1. Download Poetry</summary>\n<p>\n\n```bash\nmake download-poetry\n```\n\n</p>\n</details>\n\n<details>\n<summary>2. Install all dependencies and pre-commit hooks</summary>\n<p>\n\n```bash\nmake install\n```\n\nIf you do not want to install pre-commit hooks, run the command with the NO_PRE_COMMIT flag:\n\n```bash\nmake install NO_PRE_COMMIT=1\n```\n\n</p>\n</details>\n\n<details>\n<summary>3. Check the security of your code</summary>\n<p>\n\n```bash\nmake check-safety\n```\n\nThis command launches a `Poetry` and `Pip` integrity check as well as identifies security issues with `Safety` and `Bandit`. By default, the build will not crash if any of the items fail. But you can set `STRICT=1` for the entire build, or you can configure strictness for each item separately.\n\n```bash\nmake check-safety STRICT=1\n```\n\nor only for `safety`:\n\n```bash\nmake check-safety SAFETY_STRICT=1\n```\n\nmultiple\n\n```bash\nmake check-safety PIP_STRICT=1 SAFETY_STRICT=1\n```\n\n> List of flags for `check-safety` (can be set to `1` or `0`): `STRICT`, `POETRY_STRICT`, `PIP_STRICT`, `SAFETY_STRICT`, `BANDIT_STRICT`.\n\n</p>\n</details>\n\n<details>\n<summary>4. Check the codestyle</summary>\n<p>\n\nThe command is similar to `check-safety` but to check the code style, obviously. It uses `Black`, `Darglint`, `Isort`, and `Mypy` inside.\n\n```bash\nmake check-style\n```\n\nIt may also contain the `STRICT` flag.\n\n```bash\nmake check-style STRICT=1\n```\n\n> List of flags for `check-style` (can be set to `1` or `0`): `STRICT`, `BLACK_STRICT`, `DARGLINT_STRICT`, `ISORT_STRICT`, `MYPY_STRICT`.\n\n</p>\n</details>\n\n<details>\n<summary>5. Run all the codestyle formaters</summary>\n<p>\n\nCodestyle uses `pre-commit` hooks, so ensure you\'ve run `make install` before.\n\n```bash\nmake codestyle\n```\n\n</p>\n</details>\n\n<details>\n<summary>6. Run tests</summary>\n<p>\n\n```bash\nmake test\n```\n\n</p>\n</details>\n\n<details>\n<summary>7. Run all the linters</summary>\n<p>\n\n```bash\nmake lint\n```\n\nthe same as:\n\n```bash\nmake test && make check-safety && make check-style\n```\n\n> List of flags for `lint` (can be set to `1` or `0`): `STRICT`, `POETRY_STRICT`, `PIP_STRICT`, `SAFETY_STRICT`, `BANDIT_STRICT`, `BLACK_STRICT`, `DARGLINT_STRICT`, `ISORT_STRICT`, `MYPY_STRICT`.\n\n</p>\n</details>\n\n<details>\n<summary>8. Build docker</summary>\n<p>\n\n```bash\nmake docker\n```\n\nwhich is equivalent to:\n\n```bash\nmake docker VERSION=latest\n```\n\nMore information [here](https://github.com/matthijs2704/samsung-mdc-ctl/tree/main/docker).\n\n</p>\n</details>\n\n<details>\n<summary>9. Cleanup docker</summary>\n<p>\n\n```bash\nmake clean_docker\n```\n\nor to remove all build\n\n```bash\nmake clean\n```\n\nMore information [here](https://github.com/matthijs2704/samsung-mdc-ctl/tree/main/docker).\n\n</p>\n</details>\n\n## 📈 Releases\n\nYou can see the list of available releases on the [GitHub Releases](https://github.com/matthijs2704/samsung-mdc-ctl/releases) page.\n\nWe follow [Semantic Versions](https://semver.org/) specification.\n\nWe use [`Release Drafter`](https://github.com/marketplace/actions/release-drafter). As pull requests are merged, a draft release is kept up-to-date listing the changes, ready to publish when you’re ready. With the categories option, you can categorize pull requests in release notes using labels.\n\nFor Pull Request this labels are configured, by default:\n\n|               **Label**               |  **Title in Releases**  |\n| :-----------------------------------: | :---------------------: |\n|       `enhancement`, `feature`        |       🚀 Features       |\n| `bug`, `refactoring`, `bugfix`, `fix` | 🔧 Fixes & Refactoring  |\n|       `build`, `ci`, `testing`        | 📦 Build System & CI/CD |\n|              `breaking`               |   💥 Breaking Changes   |\n|            `documentation`            |    📝 Documentation     |\n|            `dependencies`             | ⬆️ Dependencies updates |\n\nYou can update it in [`release-drafter.yml`](https://github.com/matthijs2704/samsung-mdc-ctl/blob/main/.github/release-drafter.yml).\n\nGitHub creates the `bug`, `enhancement`, and `documentation` labels for you. Dependabot creates the `dependencies` label. Create the remaining labels on the Issues tab of your GitHub repository, when you need them.\n\n## 🛡 License\n\n[![License](https://img.shields.io/github/license/matthijs2704/samsung-mdc-ctl)](https://github.com/matthijs2704/samsung-mdc-ctl/blob/main/LICENSE)\n\nThis project is licensed under the terms of the `MIT` license. See [LICENSE](https://github.com/matthijs2704/samsung-mdc-ctl/blob/main/LICENSE) for more details.\n\n## 📃 Citation\n\n```\n@misc{samsung-mdc-ctl,\n  author = {matthijs2704},\n  title = {Awesome `samsung-mdc-ctl` is a Python cli/package created with https://github.com/TezRomacH/python-package-template},\n  year = {2021},\n  publisher = {GitHub},\n  journal = {GitHub repository},\n  howpublished = {\\url{https://github.com/matthijs2704/samsung-mdc-ctl}}\n}\n```\n\n## Credits\n\nThis project was generated with [`python-package-template`](https://github.com/TezRomacH/python-package-template).\n',
    'author': 'matthijs2704',
    'author_email': 'matthijs@logemann.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/matthijs2704/samsung-mdc-ctl',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# type: ignore[attr-defined]

from typing import Optional

import logging
import random
import time
from enum import Enum

import typer
from rich.console import Console
from samsung_mdc_ctl import __version__
from samsung_mdc_ctl.example import hello
from samsung_mdc_ctl.helpers.connection import MDCConnection
from samsung_mdc_ctl.helpers.exceptions import NakReceived
from samsung_mdc_ctl.mdc_display import MDCDisplay
from samsung_mdc_ctl.protocol.commands import Command
from samsung_mdc_ctl.protocol.power import DisplayPower
from samsung_mdc_ctl.protocol.sources import InputSource


class Color(str, Enum):
    white = "white"
    red = "red"
    cyan = "cyan"
    magenta = "magenta"
    yellow = "yellow"
    green = "green"


app = typer.Typer(
    name="samsung-mdc-ctl",
    help="Awesome `samsung-mdc-ctl` is a Python cli/package created with https://github.com/TezRomacH/python-package-template",
    add_completion=False,
)
console = Console()


def version_callback(value: bool):
    """Prints the version of the package."""
    if value:
        console.print(
            f"[yellow]samsung-mdc-ctl[/] version: [bold blue]{__version__}[/]"
        )
        raise typer.Exit()


@app.command(name="")
def main(
    name: str = typer.Option(..., help="Name of person to greet."),
    color: Optional[Color] = typer.Option(
        None,
        "-c",
        "--color",
        "--colour",
        case_sensitive=False,
        help="Color for name. If not specified then choice will be random.",
    ),
    version: bool = typer.Option(
        None,
        "-v",
        "--version",
        callback=version_callback,
        is_eager=True,
        help="Prints the version of the samsung-mdc-ctl package.",
    ),
):
    """Prints a greeting for a giving name."""
    # logger.setLevel()
    greeting: str = hello(name)

    logging.basicConfig(encoding="utf-8", level=logging.DEBUG)

    display = MDCDisplay(host="192.168.3.153", deviceId=1)
    # print(display.getModelNumber().__dict__)
    # print(display.getSWVersion())
    # print(display.getStatus().__dict__)
    # print(display.getPower())
    # print(display.getMute())
    # print(display.getVolume())
    # display.setMute(False)
    # display.setPower(DisplayPower.POWER_ON)
    # display.setVolume(10)
    try:
        print(display.getPower())
        print(display.setPower(DisplayPower.POWER_OFF))
        time.sleep(2)
        print(display.getPower())
        # time.sleep(10)
        print(display.setPower(DisplayPower.POWER_ON))
        print(display.getPower())
        # resp = display.connection.send(cmd=Command.PIP_STATUS, data=[])
        # print(f"Resp: %s" % resp)
        # print(f"Resp Data: %s" % resp.__dict__)
    except NakReceived as e:
        print(f"NAK: %s" % e.args)

    # try:
    #     print(display.getInputSource())
    #     print(display.setInputSource(InputSource.HDMI_1))
    # except NakReceived as e:
    #     print(e.args)

    # print(display.connection.send(Command.OUTDOOR_MODE, [0x81]).__dict__)
    # console.print(f"[bold magenta]Got connection to the Samsung display![/]")
    # print(connection.send(Command.STATUS))
    # print(connection.send(Command.MUTE, [0x0]))
    # connection.close()



from samsung_mdc_ctl.protocol.sources import InputSource


class Status:
    """ Representation of the status message """

    def __init__(self, payload: bytes) -> None:
        self.power = payload[0] == 1
        self.volume = payload[1]
        self.mute = payload[2] == 1
        self.input = InputSource(payload[3])
        self.aspect = payload[4]
        pass
